//+------------------------------------------------------------------+
//|                                                     Messages.mqh |
//|                                                Ernestas Kvedaras |
//|                                           https://forex-blog.pro |
//+------------------------------------------------------------------+
#property library
#property copyright     "Ernestas Kvedaras"
#property link          "https://forex-blog.pro"
#property version       "1.00"
#property description   "Various helper functions for sending messages of different types to different sources"
#property strict
//+------------------------------------------------------------------+
//| Message to text definitions                                      |
//+------------------------------------------------------------------+

#define MESSAGE_TEXT_DEBUG          "DEBUG"
#define MESSAGE_TEXT_NOTICE         "NOTICE"
#define MESSAGE_TEXT_WARNING        "WARNING"
#define MESSAGE_TEXT_ERROR          "ERROR"
#define MESSAGE_TEXT_SUCCESS        "SUCCESS"
#define MESSAGE_TEXT_INFO           "INFO"
#define MESSAGE_TEXT_SEPARATOR      " | "
#define MESSAGE_MAIL_SUBJECT_PREFIX "My EA: "
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| Possible message types                                           |
//+------------------------------------------------------------------+
enum ENUM_MESSAGE_TYPE
  {
   MESSAGE_NONE,     // Message type: not defined   
   MESSAGE_DEBUG,    // Message type: DEBUG
   MESSAGE_NOTICE,   // Message type: NOTICE
   MESSAGE_WARNING,  // Message type: WARNING
   MESSAGE_ERROR,    // Message type: ERROR
   MESSAGE_SUCCESS,  // Message type: SUCCESS
   MESSAGE_INFO,     // Message type: INFO
  };
//+------------------------------------------------------------------+

extern bool messageDebugMode=false; // Debug messages
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| DEBUG                                                            |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print DEBUG message if debug mode is enabled                     |
//+------------------------------------------------------------------+
void debug(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   if(!messageDebugMode)
      return;

   Print(formatMessage(message,MESSAGE_DEBUG,secondType));
  }
//+------------------------------------------------------------------+
//| Alert DEBUG messsage if debug mode is enabled                    |
//+------------------------------------------------------------------+
void debugAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   if(!messageDebugMode)
      return;

   Alert(formatMessage(message,MESSAGE_DEBUG,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with DEBUG messsage if debug mode is enabled    |
//+------------------------------------------------------------------+
void debugMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   if(!messageDebugMode)
      return;

   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_DEBUG));
  }
//+------------------------------------------------------------------+
//| Send DEBUG notification if debug mode is enabled                 |
//+------------------------------------------------------------------+
void debugNotification(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   if(!messageDebugMode)
      return;

   SendNotification(formatMessage(message,MESSAGE_DEBUG,secondType));
  }
//+------------------------------------------------------------------+
//| Send DEBUG mail if debug mode is enabled                         |
//+------------------------------------------------------------------+
void debugMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   if(!messageDebugMode)
      return;

   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_DEBUG),formatMessage(message,MESSAGE_DEBUG,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| NOTICE                                                           |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print NOTICE message                                             |
//+------------------------------------------------------------------+
void notice(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Print(formatMessage(message,MESSAGE_NOTICE,secondType));
  }
//+------------------------------------------------------------------+
//| Alert NOTICE messsage                                            |
//+------------------------------------------------------------------+
void noticeAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Alert(formatMessage(message,MESSAGE_NOTICE,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with NOTICE messsage                            |
//+------------------------------------------------------------------+
void noticeMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_NOTICE));
  }
//+------------------------------------------------------------------+
//| Send NOTICE notification                                         |
//+------------------------------------------------------------------+
void noticeNotification(const string message,const ENUM_MESSAGE_TYPE secondType=NULL) export
  {
   SendNotification(formatMessage(message,MESSAGE_NOTICE,secondType));
  }
//+------------------------------------------------------------------+
//| Send NOTICE mail                                                 |
//+------------------------------------------------------------------+
void noticeMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_NOTICE),formatMessage(message,MESSAGE_NOTICE,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| WARNING                                                          |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print WARNING message                                            |
//+------------------------------------------------------------------+
void warning(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Print(formatMessage(message,MESSAGE_WARNING,secondType));
  }
//+------------------------------------------------------------------+
//| Alert WARNING messsage                                           |
//+------------------------------------------------------------------+
void warningAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Alert(formatMessage(message,MESSAGE_WARNING,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with WARNING messsage                           |
//+------------------------------------------------------------------+
void warningMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_WARNING));
  }
//+------------------------------------------------------------------+
//| Send WARNING notification                                        |
//+------------------------------------------------------------------+
void warningNotification(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendNotification(formatMessage(message,MESSAGE_WARNING,secondType));
  }
//+------------------------------------------------------------------+
//| Send WARNING mail                                                 |
//+------------------------------------------------------------------+
void warningMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_WARNING),formatMessage(message,MESSAGE_WARNING,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| ERROR                                                            |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print ERROR message                                              |
//+------------------------------------------------------------------+
void error(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Print(formatMessage(message,MESSAGE_ERROR,secondType));
  }
//+------------------------------------------------------------------+
//| Alert ERROR messsage                                             |
//+------------------------------------------------------------------+
void errorAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Alert(formatMessage(message,MESSAGE_ERROR,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with ERROR messsage                             |
//+------------------------------------------------------------------+
void errorMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_ERROR));
  }
//+------------------------------------------------------------------+
//| Send ERROR notification                                          |
//+------------------------------------------------------------------+
void errorNotification(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendNotification(formatMessage(message,MESSAGE_ERROR,secondType));
  }
//+------------------------------------------------------------------+
//| Send ERROR mail                                                 |
//+------------------------------------------------------------------+
void errorMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_ERROR),formatMessage(message,MESSAGE_ERROR,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| SUCCESS                                                          |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print SUCCESS message                                            |
//+------------------------------------------------------------------+
void success(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Print(formatMessage(message,MESSAGE_SUCCESS,secondType));
  }
//+------------------------------------------------------------------+
//| Alert SUCCESS messsage                                           |
//+------------------------------------------------------------------+
void successAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Alert(formatMessage(message,MESSAGE_SUCCESS,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with SUCCESS messsage                           |
//+------------------------------------------------------------------+
void successMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_SUCCESS));
  }
//+------------------------------------------------------------------+
//| Send SUCCESS notification                                        |
//+------------------------------------------------------------------+
void successNotification(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendNotification(formatMessage(message,MESSAGE_SUCCESS,secondType));
  }
//+------------------------------------------------------------------+
//| Send SUCCESS mail                                                |
//+------------------------------------------------------------------+
void successMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_SUCCESS),formatMessage(message,MESSAGE_SUCCESS,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| INFO                                                             |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Print INFO message                                               |
//+------------------------------------------------------------------+
void info(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Print(formatMessage(message,MESSAGE_INFO,secondType));
  }
//+------------------------------------------------------------------+
//| Alert INFO messsage                                              |
//+------------------------------------------------------------------+
void infoAlert(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   Alert(formatMessage(message,MESSAGE_INFO,secondType));
  }
//+------------------------------------------------------------------+
//| Show message box with INFO messsage                              |
//+------------------------------------------------------------------+
void infoMessageBox(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   MessageBox(formatMessage(message,secondType),typeToStr(MESSAGE_INFO));
  }
//+------------------------------------------------------------------+
//| Send INFO notification                                           |
//+------------------------------------------------------------------+
void infoNotification(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendNotification(formatMessage(message,MESSAGE_INFO,secondType));
  }
//+------------------------------------------------------------------+
//| Send INFO mail                                                |
//+------------------------------------------------------------------+
void infoMail(const string message,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE) export
  {
   SendMail(MESSAGE_MAIL_SUBJECT_PREFIX+typeToStr(MESSAGE_INFO),formatMessage(message,MESSAGE_INFO,secondType));
  }
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//| PRIVATE FUNCTIONS                                                |
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Format given message and print                                   |
//+------------------------------------------------------------------+
string formatMessage(const string message,const ENUM_MESSAGE_TYPE type=MESSAGE_NONE,const ENUM_MESSAGE_TYPE secondType=MESSAGE_NONE)
  {
   string prefix="";

   if(type!=NULL)
      prefix+=typeToStr(type)+MESSAGE_TEXT_SEPARATOR;

   if(secondType!=NULL)
      prefix+=typeToStr(secondType)+MESSAGE_TEXT_SEPARATOR;

   return prefix + message;
  }
//+------------------------------------------------------------------+
//| Convenrt ENUM_MESSAGE_TYPE to string                             |
//+------------------------------------------------------------------+
string typeToStr(const ENUM_MESSAGE_TYPE type)
  {
   switch(type)
     {
      case MESSAGE_DEBUG:     return MESSAGE_TEXT_DEBUG;
      case MESSAGE_NOTICE:    return MESSAGE_TEXT_NOTICE;
      case MESSAGE_WARNING:   return MESSAGE_TEXT_WARNING;
      case MESSAGE_ERROR:     return MESSAGE_TEXT_ERROR;
      case MESSAGE_SUCCESS:   return MESSAGE_TEXT_SUCCESS;
      default:                return MESSAGE_TEXT_INFO;
     }
  }
//+------------------------------------------------------------------+
